@@ -1,14 +1,14 @@
 # -*- perl -*-
 
 #
-# $Id: HistEntry.pm,v 1.23 2001/04/29 18:48:23 eserte Exp $
+# $Id: HistEntry.pm,v 1.27 2003/08/01 15:55:53 eserte Exp $
 # Author: Slaven Rezic
 #
-# Copyright  1997, 2000, 2001 Slaven Rezic. All rights reserved.
+# Copyright  1997, 2000, 2001, 2003 Slaven Rezic. All rights reserved.
 # This package is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 #
-# Mail: mailto:eserte@cs.tu-berlin.de
+# Mail: slaven@rezic.de
 # WWW:  http://www.cs.tu-berlin.de/~eserte/
 #
 
@@ -17,7 +17,7 @@ require Tk;
 use strict;
 use vars qw($VERSION);
 
-$VERSION = '0.37';
+$VERSION = '0.41';
 
 sub addBind {
     my $w = shift;
@@ -123,6 +123,7 @@ sub historyUpdate {
     my $w = shift;
     $w->_update($w->privateData->{'history'}->[$w->privateData->{'historyindex'}]);
     $w->_entry->icursor('end'); # suggestion by Jason Smith <smithj4@rpi.edu>
+    $w->_entry->xview('insert');
 }
 
 sub historyUp {
@@ -276,7 +277,11 @@ sub KeyPress {
 
     my $text = $e->get;
     ###Grab test from entry upto cursor
-    (my $typedtext = $text) =~ s/^(.{$cursor}).*/$1/;
+    (my $typedtext = $text) =~ s/^(.{$cursor})(.*)/$1/;
+    if ($2 ne "") {
+	###text after cursor, do not use matching
+	return;
+    }
 
     if ($cursor == 0 || $text eq '') {
 	###No text before cursor, reset list
@@ -323,6 +328,12 @@ use vars qw(@ISA);
 #use base qw(Tk::Derived Tk::Entry Tk::HistEntry);
 Construct Tk::Widget 'SimpleHistEntry';
 
+sub CreateArgs {
+    my($package, $parent, $args) = @_;
+    $args->{-class} = "SimpleHistEntry" unless exists $args->{-class};
+    $package->SUPER::CreateArgs($parent, $args);
+}
+
 sub Populate {
     my($w, $args) = @_;
 
@@ -345,6 +356,7 @@ sub Populate {
        -limit   => ['PASSIVE',  'limit',   'Limit',   undef],
        -match   => ['PASSIVE',  'match',   'Match',   0],
        -case    => ['PASSIVE',  'case',    'Case',    1],
+       -history => ['METHOD'],
       );
 
     $w;
@@ -359,6 +371,12 @@ use vars qw(@ISA);
 #use base qw(Tk::Derived Tk::BrowseEntry Tk::HistEntry);
 Construct Tk::Widget 'HistEntry';
 
+sub CreateArgs {
+    my($package, $parent, $args) = @_;
+    $args->{-class} = "HistEntry" unless exists $args->{-class};
+    $package->SUPER::CreateArgs($parent, $args);
+}
+
 sub Populate {
     my($w, $args) = @_;
 
@@ -384,6 +402,8 @@ sub Populate {
     $w->{start} = 0;
     $w->{end} = 0;
 
+    my $entry = $w->Subwidget('entry');
+
     $w->ConfigSpecs
       (-command => ['CALLBACK', 'command', 'Command', undef],
        -auto    => ['PASSIVE',  'auto',    'Auto',    0],
@@ -392,11 +412,12 @@ sub Populate {
        -limit   => ['PASSIVE',  'limit',   'Limit',   undef],
        -match   => ['PASSIVE',  'match',   'Match',   0],
        -case    => ['PASSIVE',  'case',    'Case',    1],
+       -history => ['METHOD'],
       );
 
-    $w->Delegates('delete' => $w->Subwidget('entry'),
-		  'get'    => $w->Subwidget('entry'),
-		  'insert' => $w->Subwidget('entry'),
+    $w->Delegates('delete' => $entry,
+		  'get'    => $entry,
+		  'insert' => $entry,
 		 );
 
     $w;
@@ -606,13 +627,13 @@ found in the t and examples directories of the source distribution.
 If you like to not depend on the installation of Tk::HistEntry, you
 can write something like this:
 
-    $Entry = "SimpleHistEntry";
+    $Entry = "Entry"; # default Entry widget
     eval {
         # try loading the module, otherwise $Entry is left to the value "Entry"
 	require Tk::HistEntry;
 	$Entry = "SimpleHistEntry";
     };
-    $entry = $status_frame->$Entry(-textvariable => \$res)->pack;
+    $entry = $mw->$Entry(-textvariable => \$res)->pack;
     $entry->bind("<Return>" => sub {
                                    # check whether the historyAdd method is
 		                   # known to the widget
@@ -621,6 +642,27 @@ can write something like this:
 				   }
                                });
 
+In this approach the history lives in an array variable. Here the
+entry widget does not need to be permanent, that is, it is possible to
+destroy the containing window and restore the history again:
+
+    $Entry = "Entry";
+    eval {
+	require Tk::HistEntry;
+        $Entry = "HistEntry";
+    };
+    $entry = $mw->$Entry(-textvariable => \$res)->pack;
+    if ($entry->can('history') && @history) {
+	$entry->history(\@history);
+    }
+
+    # Later, after clicking on a hypothetical "Ok" button:
+    if ($res ne "" && $entry->can('historyAdd')) {
+        $entry->historyAdd($res);
+	@history = $entry->history;
+    }
+
+
 =head1 BUGS/TODO
 
  - C-s/C-r do not work as nice as in gnu readline
@@ -629,7 +671,7 @@ can write something like this:
 
 =head1 AUTHOR
 
-Slaven Rezic <eserte@cs.tu-berlin.de>
+Slaven Rezic <slaven@rezic.de>
 
 =head1 CREDITS
 
@@ -640,7 +682,7 @@ code is stolen from Tk::IntEntry by Dave Collins
 
 =head1 COPYRIGHT
 
-Copyright (c) 1997, 2000, 2001 Slaven Rezic. All rights reserved.
+Copyright (c) 1997, 2000, 2001, 2003 Slaven Rezic. All rights reserved.
 This package is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
 
